FlexBar2 = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceModuleCore-2.0", "AceDebug-2.0")
FlexBar2.OO = AceLibrary("AceOO-2.0");
FlexBar2.Waterfall = AceLibrary("Waterfall-1.0");

FlexBar2:RegisterDB("FlexBar2_DB");
FlexBar2:RegisterDefaults('profile', {
	Buttons = {},
});

FlexBar2:RegisterChatCommand({"/flexbar", "/flexbar2", "/fb", "/fb2"}, 
{
    type = 'execute',
    desc = "Show configuration screen",
    func = function()
        FlexBar2.Waterfall:Open("FlexBar2");
    end
});

function FlexBar2:OnInitialize()
    self:SetDebugging(true);
	self.ModuleOnEnableList = {};
	self.ButtonList = {};
	self.ButtonMixinList = {};
	self.GroupMixinList = {};
	self.ButtonActivatorList = {};
	self.GroupActivatorList = {};
	self.ExtraOptionsTableList = {};
	self.Buttons = {};
	self:Debug("Searching for modules");
    for Name, Module in self:IterateModules() do
		self:Debug("     Found Module: " .. Name);
		if(type(Module.ButtonMixin) == "table") then
			table.insert(self.ButtonMixinList, Module.ButtonMixin);
			self:Debug("          Found ButtonMixin");
			if(type(Module.ButtonMixin.Activate) == "function") then
				self:Debug("               Found Activator");
				table.insert(FlexBar2.ButtonActivatorList, Module.ButtonMixin.Activate);
			end
		end
		if(type(Module.GroupMixin) == "table") then
			self:Debug("          Found GroupMixin: ");
			self.GroupMixinList[Name] = Module.GroupMixin;
			if(type(Module.GroupMixin.Activate) == "function") then
				self:Debug("               Found Activator");
				table.insert(self.GroupActivatorList, Module.GroupMixin.Activate);
			end
		end
		if(type(Module.OnEnable) == "function") then
			self:Debug("          Found OnEnable handler");
			self.ModuleOnEnableList[Name] = Module.OnEnable;
		end
		if(type(Module.AceOptions) == "table") then
			self:Debug("          Found AceOptions");
			table.insert(self.ExtraOptionsTableList, Module.AceOptions);
		end
	end
	
	self.ButtonSuperClass = self.OO.Class(unpack(self.ButtonMixinList));
		function self.ButtonSuperClass.prototype:init(Name, ...)
			FlexBar2.ButtonSuperClass.super.prototype.init(self);
			FlexBar2:Debug("          Initializing");
			if(type(Name) ~= "string") then
				FlexBar2:Debug("               Failed, invalid name");
				return false;
			end
			self.InfoTable = FlexBar2.db.profile.Buttons[Name];
			FlexBar2.Buttons[Name] = self;
			table.insert(FlexBar2.ButtonList, Name);
			for _, Func in ipairs(FlexBar2.ButtonActivatorList) do
				Func(self, ...);
			end
		end
	self.GroupSuperClass = self.OO.Class(unpack(self.GroupMixinList));
		function self.GroupSuperClass.prototype:init(...)
			FlexBar2:Debug("          Initializing");
			FlexBar2.GroupSuperClass.super.prototype.init(self);
			for _, Func in ipairs(FlexBar2.GroupActivatorList) do
				Func(self, ...);
			end
		end
end

function FlexBar2:Create(Name)
	if(type(FlexBar2.db.profile.Buttons[Name]) ~= "table") then
		FlexBar2.db.profile.Buttons[Name] = { Generate = true};
		FlexBar2:Generate(Name);
	else
		FlexBar2:Print("Button already exists");
	end
end

function FlexBar2:Generate(Name)
	self:Debug(Name);
	if(self.Buttons[Name] ~= nil) then
		self:Debug("          Error: Button already exists, aborting");
		return false;
	end
	self.ButtonSuperClass:new(Name);
	return true;
end

function FlexBar2:OnEnable()
	local Waterfall = self.Waterfall;
	self:Debug("Setting up Waterfall");
	local WaterfallTree = 
	{
		{
			text = "General Settings",
			id = "gnrlset",
			isOpen = true,
		},
		{
			text = "Button Control",
			id = "buttonctrl",
			isOpen = false,
		},
		{
			text = "Group Control",
			id = "groupctrl",
			isOpen = false,
		},
	};
	local buttonctrl_CreateTemp;
	local buttonctrl_EditSave = false;
	local buttonctrl_EditTemp;
	local function BuildWaterfallTree(Id)
		if not Id then return ""; end
		if(Id == "gnrlset") then			
		elseif(Id == "buttonctrl") then
			if(not buttonctrl_EditSave) then
				-- Create Button --
				Waterfall:AddControl(
				"type", "label",
				"text", "Create Button: ");
				
				Waterfall:AddControl(
				"type","textbox",
				"width",200,
				"fullRefresh", true,
				"noNewLine", true,
				"setFunc", function() FlexBar2:Create(buttonctrl_CreateTemp); end,
				"getFunc", function() return ""; end,
				"setArg1", FlexBar2,
				"changedFunc", function(ButtonName) buttonctrl_CreateTemp = ButtonName; end);
				
				Waterfall:AddControl(
				"type","button",
				"text","Create",
				"fullRefresh", true,
				"noNewLine",true,
				"width",80,
				"causeRefresh",true,
				"execFunc", function() FlexBar2:Create(buttonctrl_CreateTemp); end);
				
				-- Edit Button --
				Waterfall:AddControl(
				"type", "label",
				"text", "Edit Button: ");
				
				Waterfall:AddControl(
				"type","textbox",
				"width",200,
				"noNewLine", true,
				"fullRefresh", true,
				"setFunc", function(ButtonName) buttonctrl_EditSave = ButtonName end,
				"changedFunc", function(ButtonName) buttonctrl_EditTemp = ButtonName end);
				
				Waterfall:AddControl(
				"type","button",
				"text","Edit",
				"noNewLine",true,
				"fullRefresh", true,
				"width",80,
				"execFunc", function() buttonctrl_EditSave = buttonctrl_EditTemp end);
				return "Basic Button Control";
			else
				-- Delete Button --
				Waterfall:AddControl(
				"type", "button",
				"text","Delete Button",
				"fullRefresh", true,
				"width", 100,
				"execFunc", function() FlexBar2.Delete(buttonctrl_EditSave); end);
				
				-- Save Button --
				Waterfall:AddControl(
				"type", "button",
				"text","cancel changes",
				"fullRefresh", true,
				"width", 130,
				"execFunc", function() buttonctrl_EditSave = false; end);
				
				-- Cancel Button --
				Waterfall:AddControl(
				"type", "button",
				"text","Confirm changes",
				"fullRefresh", true,
				"noNewLine", true,
				"width", 130,
				"execFunc", function() buttonctrl_EditSave = false; end,
				"execArg1");
				return "Editting button settings: " .. buttonctrl_EditSave;
			end
		elseif(Id == "groupctrl") then
			
		end
	end
	Waterfall:Register('FlexBar2',
	'title', 'FlexBar2 Config',
	'width', 800,
	'height', 800,
	'tree', WaterfallTree,
	'children', BuildWaterfallTree);
	
	self:Debug("Generating buttons");
	for Name, InfoTable in pairs(self.db.profile.Buttons) do
		if(InfoTable.Generate == true) then
			self:Debug("     Generating " .. Name);
			self:Generate(Name);
		end
	end
	for _, Func in ipairs(self.ModuleOnEnableList) do
		Func(self);
	end
end

function FlexBar2:OnDisable()
    
end

function FlexBar2:OnModuleCreated(Name, ref)

end
